/*************************************************************************************
 1. A mouse left-drag will rotate the camera around the subject.
 2. A right drag will zoom in and out. 
 3. A middle-drag (command-left-drag on mac) will pan. 
 4. A double-click restores the camera to its original position. 
 5. The shift key constrains rotation and panning to one axis or the other.
 *************************************************************************************/

import peasy.*;
PeasyCam cam; 

Table table;
int rowNums;
ArrayList<Book> books = new ArrayList<Book>();

ArrayList<axiss_array> axis_a = new ArrayList<axiss_array>();
float[][] books_points;
float[][] axisArray;

boolean digiSwitch = true, virtSwitch = true, cyberSwitch = true;
int transp = 45;

void setup() {
  size(1600, 900, P3D);
  float fov = PI/4; // default PI/3
  float aspect = float(width)/float(height);
  float zNear = 1;
  float zFar = 10000;
  textMode(SHAPE);
  textSize(30);
  perspective(fov, aspect, zNear, zFar);

  cam = new PeasyCam(this, 1000); // larger the number, further the camera distance to the object
  table = loadTable("newitemcheckoutcheckins.csv", "header");
  rowNums = table.getRowCount();
  cam.setWheelScale(5);
  float[][] axisArray = new float[rowNums][5];

  for (int i = 0; i<rowNums; i++) {
    // Title (Color)
    String title = table.getString(i, 0);
    // Item Number (Size)
    int itemNum = table.getInt(i, 1);
    // Checkout
    String checkout = table.getString(i, 2);
    String coutTime = table.getString(i, 3);
    // Checkin
    String checkin = table.getString(i, 4);
    books.add(new Book(title, itemNum, checkout, coutTime, checkin));

    float timeAxis = map(timeParser(books.get(i).coutTime), 600, 1200, 0, 2400);
    float dateAxis = map(dateParser(books.get(i).checkout), 0, 3650, 0, 7300);
    float coutBoxHeight = (dateParser(books.get(i).checkin) - dateParser(books.get(i).checkout));
    axisArray[i][1] = timeAxis;
    axisArray[i][2] = coutBoxHeight;
    axisArray[i][3] = dateAxis;
    int itemBoxSize = ((books.get(i).itemNum)/(1000000))*6;
    axisArray[i][4] = itemBoxSize;
    color c =  titleDetermine(books.get(i).title);
    axis_a.add(new axiss_array(timeAxis, coutBoxHeight, dateAxis, itemBoxSize, c));
  }
}


void draw() {

  //  //transp = 45;
  //  //if (axis_a.get(i).c == #ff0000 && cyberSwitch == false)
  //  //{
  //  //  transp = 0;
  //  //}

  //  //if (axis_a.get(i).c == #00FF00 && virtSwitch == false)
  //  //{
  //  //  transp = 0;
  //  //}

  //  //if (axis_a.get(i).c == #0000ff && digiSwitch == false)
  //  //{
  //  //  transp = 0;
  //  //} 

  //  //if (cyberSwitch == false && axis_a.get(i).c == #ffff00 || virtSwitch == false && axis_a.get(i).c == #ffff00)
  //  //{
  //  //  transp = 0;
  //  //}

  //  //if ( cyberSwitch == false &&  axis_a.get(i).c == #ff00ff || digiSwitch == false  && axis_a.get(i).c == #ff00ff)
  //  //{
  //  //  transp = 0;
  //  //}

  //  //if (virtSwitch == false && axis_a.get(i).c == #00FFFF || digiSwitch == false && axis_a.get(i).c == #00FFFF )
  //  //{
  //  //  transp = 0;
  //  //} else if (cyberSwitch == true && virtSwitch == true && digiSwitch == true)
  //  //{
  //  //  transp = 45;
  //  //}

  //  // Drawing the boxes
  //  //pushMatrix();
  //  //  if (transp == 0) {
  //  //  stroke(axis_a.get(i).c, transp);
  //  //} else {
  //  //  stroke(axis_a.get(i).c, 100);
  //  //}

  //  //float mouseObjectDistance = sq(mouseX-screenX(0, 0, 0))+sq(mouseY-screenY(0, 0, 0)); 
  //  //if (mouseObjectDistance < 20)
  //  //{
  //  //  stroke(c, 255);
  //  //  point(0, 0, 0);
  //  //  stroke(0, 0, 0, 0);
  //  //  fill(255, 255, 255);
  //  //  textMode(SHAPE);
  //  //  textSize(3);
  //  //  text(books.get(i).title.toUpperCase(), 0, 0);
  //  //}

  background(0);
  
  for (int n=0; n<rowNums; n++)
  {
    pushMatrix();
    fill(axis_a.get(n).c, 45);
    stroke(axis_a.get(n).c, 100);
    //strokeWeight(5);
    translate(axis_a.get(n).timeAxis, -axis_a.get(n).coutBoxHeight/2, -(axis_a.get(n).dateAxis));
    box(axis_a.get(n).itemBoxSize, axis_a.get(n).coutBoxHeight, axis_a.get(n).itemBoxSize);
    //line(axis_a.get(n).timeAxis, 0, -(axis_a.get(n).dateAxis), axis_a.get(n).timeAxis, -axis_a.get(n).coutBoxHeight/2, -(axis_a.get(n).dateAxis));
    popMatrix();
  }

  strokeWeight(1);
  textSize(30);
  for (int j = 0; j < 9400; j += 730)
  {
    pushMatrix();
    translate(1200, 0, -j);
    stroke(255, 70);
    fill(255, 70);
    box(2400, 1, 1);
    popMatrix();
  }

  for (int t = 0; t < 2600; t += 200)
  {
    pushMatrix();
    translate(t, 0, -4500);
    stroke(255, 70);
    fill(255, 70);
    box(1, 1, 9000);
    popMatrix();
  }

  int yearNum = 2004;

  for (int j = 0; j < 9400; j += 730)
  {
    fill(255);
    text(str(yearNum), -80, 0, -j);
    yearNum ++;
  }

  int timeNum = 6;

  for (int t = 0; t < 2600; t += 200)
  {
    text(str(timeNum)+":00", t-30, 20, 30);
    timeNum ++;
  }

  yearNum = 2004;
  for (int j = 0; j < 9400; j += 730)
  {
    fill(255);
    text(str(yearNum), 2400, 0, -j);
    yearNum ++;
  }

  timeNum = 6;

  for (int t = 0; t < 2600; t += 200)
  {
    text(str(timeNum)+":00", t-30, 20, -9250);
    timeNum ++;
  }

  textSize(180);
  text("DIGITAL SKYLINE", 500, -500, -9250);
}